// Clase Fecha - Archivo de cabecera "fecha.h"

#ifndef fecha_h // Evitar inclusiones mltiples
#define fecha_h

#include <iostream>
#include <string>
using namespace std;

class Fecha {
public:
	Fecha(int = 1, int = 1, int = 1900); // Constructor (da, mes, ao)
  Fecha(const Fecha&);
  Fecha& operator=(const Fecha&);
  ~Fecha();

  int dia() const;
  int mes() const;  // Accedentes
  int anio() const;

  void dia(int);
  void mes(int);    // Mutadores
  void anio(int);

  long int operator-(Fecha) const;
    // Das transcurridos entre la fecha y la proporcionada
  Fecha operator+(long int) const; // Incremento de la fecha en das
  Fecha operator-(long int) const; // Decremento de la fecha en das
  Fecha& operator+=(long int); // Abreviatura para la suma
  Fecha& operator-=(long int); // Abreviatura para la resta
  Fecha& operator++(); // Da siguiente
  Fecha& operator++(int); // (variante postfija)
  Fecha& operator--(); // Da anterior
  Fecha& operator--(int); // (variante postfija)

  bool operator==(Fecha) const;
  bool operator!=(Fecha) const;
  bool operator<(Fecha) const;    // Operadores relacionales
  bool operator<=(Fecha) const;
  bool operator>(Fecha) const;
  bool operator>=(Fecha) const;

  bool bisiesto() const;
  string diaSemana() const; // Devuelve el da de la semana
  string completa() const; // Devuelve la fecha en formato
    // largo: <da de la semana> <da> de <nombre del mes> de <ao>
    // Por ejemplo: sabado 1 de abril de 2000
  string reducida() const; // Devuelve la fecha en formato
    // corto: <da>/<mes>/<ao> (da y mes con 2 dgitos)
    // Por ejemplo: 01/04/2000
  void leer();

private:
  static int dias[12]; // das de cada mes - atributo de clase
  int _dia, _mes, _anio;
  void valida();
  // comprueba que la fecha est dentro del intervalo vlido
  // si no es as, la pone a 01/01/1900 y lo avisa
  long int diasDesdeInicio() const;
  // dias desde el 01/01/1900 hasta la fecha
  Fecha& fechaTras(long int);
  // fecha tras pasar los dias indicados desde el 01/01/1900
};

#endif
