// Clase Hora - Implementacin "hora.cpp"

#include "hora.h"

// Mtodos privados:

void Hora::valida() {
  if(_horas < 0) _horas = 0;
  else _horas = _horas % 24;
  if(_minutos < 0) _minutos = 0;
  else _minutos = _minutos % 60;
  if(_segundos < 0) _segundos = 0;
  else _segundos = _segundos % 60;
}

void Hora::deSegundos(long int s) {
  const long int SPD = 86400; // segundos por da
  s = s % SPD;
  if(s < 0) s = SPD + s;
  _horas = s / 3600;
  _horas = _horas % 24;
  s = s % 3600;
  _minutos = s / 60;
  _segundos = s % 60;
}

long int Hora::aSegundos() const
{ return _horas * 3600 + _minutos * 60 + _segundos; }

// Forma cannica ortodoxa (y resto de constructores):

Hora::Hora(int h, int m, int s)
  : _horas(h), _minutos(m), _segundos(s) { valida(); }

Hora::Hora(const Hora& otra) {
  _horas = otra._horas;
  _minutos = otra._minutos;
  _segundos = otra._segundos;
}

Hora& Hora::operator=(const Hora& otra) {
  _horas = otra._horas;
  _minutos = otra._minutos;
  _segundos = otra._segundos;
  return *this;
}

Hora::~Hora() { }

Hora::Hora(long int seg) { deSegundos(seg); }

// Accedentes y mutadores:

void Hora::horas(int h) { _horas = h; valida(); }

void Hora::minutos(int m) { _minutos = m; valida(); }   // Mutadores

void Hora::segundos(int s) { _segundos = s; valida(); }

int Hora::horas() const { return _horas; }

int Hora::minutos() const { return _minutos; }       // Accedentes

int Hora::segundos() const { return _segundos; }

// Operadores aritmticos:

long int Hora::operator-(Hora otra) const
{ return aSegundos() - otra.aSegundos(); }

Hora Hora::operator+(long int s) const
{ return Hora(aSegundos() + s); }

Hora Hora::operator-(long int s) const
{ return Hora(aSegundos() - s); }

Hora& Hora::operator+=(long int s)
{ deSegundos(aSegundos() + s); return *this; }

Hora& Hora::operator-=(long int s)
{ deSegundos(aSegundos() - s); return *this; }

Hora& Hora::operator++()
{ deSegundos(aSegundos() + 1); return *this; }

Hora& Hora::operator++(int)
{ deSegundos(aSegundos() + 1); return *this; }

Hora& Hora::operator--()
{ deSegundos(aSegundos() - 1); return *this; }

Hora& Hora::operator--(int)
{ deSegundos(aSegundos() - 1); return *this; }

// Operadores relacionales:

bool Hora::operator==(Hora otra) const
{ return aSegundos() == otra.aSegundos(); }

bool Hora::operator!=(Hora otra) const
{ return aSegundos() != otra.aSegundos(); }

bool Hora::operator<(Hora otra) const
{ return aSegundos() < otra.aSegundos(); }

bool Hora::operator<=(Hora otra) const
{ return aSegundos() <= otra.aSegundos(); }

bool Hora::operator>(Hora otra) const
{ return aSegundos() > otra.aSegundos(); }

bool Hora::operator>=(Hora otra) const
{ return aSegundos() >= otra.aSegundos(); }

// Resto de mtodos:

string Hora::hora() const {
  string h;
  if(_horas < 10) h += "0";
  char c[3];
  itoa(_horas, c, 10);
  string tmp(c);
  h += tmp + ":";
  if(_minutos < 10) h += "0";
  itoa(_minutos, c, 10);
  tmp = c;
  h += tmp + ":";
  if(_segundos < 10) h += "0";
  itoa(_segundos, c, 10);
  tmp = c;
  h += tmp;
  return h;
}

void Hora::leer() {
  cout << "Horas (0-23): "; cin >> _horas;
  cout << "Minutos (0-59): "; cin >> _minutos;
  cout << "Segundos (0-59): "; cin >> _segundos;
  valida();
}

