// Clase Fraccion - Archivo de implementacin "fraccion.cpp"

#include <iostream>
using namespace std;
#include "fraccion.h"

void Fraccion::simplifica() { // Simplifica la fraccin
// Algoritmo de Euclides para el clculo del Mximo Comn Divisor (MCD)
//   Primer dividendo: el valor mayor de entre el numerador y el denominador
//   Primer divisor: el valor menor de entre el numerador y el denominador
//   Repetir sucesivas divisiones enteras entre dividendo y divisor
//   Cada siguiente divisin: como dividendo, el divisor anterior
//                            como divisor, el resto de la divisin anterior
//   Parar al obtener 0 como resto
//   El MCD es el divisor de la ltima divisin
// Simplificacin de la fraccin: dividir numerador y denominador por el MCD
	int dividendo, divisor, cociente, resto;
	if(_numerador==0 || _denominador==0) return;
	if(_numerador > _denominador)
	{
		dividendo = _numerador;
		divisor = _denominador;
	}
	else
	{
		dividendo = _denominador;
		divisor = _numerador;
	}
	cociente = dividendo / divisor;
	resto = dividendo % divisor;
	while(resto!=0) {
		dividendo = divisor;
		divisor = resto;
		cociente = dividendo / divisor;
		resto = dividendo % divisor;
	}
	_numerador /= divisor;
	_denominador /= divisor;
}

Fraccion::Fraccion(int n, int d) : _numerador(n), _denominador(d) {
  simplifica();
  // cout << "Construyo " << _numerador << "/" << _denominador << endl;
}

Fraccion::Fraccion(const Fraccion& otro) {
  _numerador = otro._numerador;
  _denominador = otro._denominador;
  // cout << "Construyo por copia " << _numerador << "/" << _denominador << endl;
}

Fraccion& Fraccion::operator=(const Fraccion& otro) {
  _numerador = otro._numerador;
  _denominador = otro._denominador;
  // cout << "Copio " << _numerador << "/" << _denominador << endl;
  return *this;
}

Fraccion::~Fraccion() {
  // cout << "Destruyo " << _numerador << "/" << _denominador << endl;
}

void Fraccion::numerador(int i) { _numerador = i; simplifica(); }

void Fraccion::denominador(int i) { _denominador = i; simplifica(); }

int Fraccion::numerador() const { return _numerador; }

int Fraccion::denominador() const { return _denominador; }

Fraccion Fraccion::operator+(Fraccion otro) const
{
	Fraccion tmp;
	tmp._numerador = _numerador * otro._denominador +
                  _denominador * otro._numerador;
	tmp._denominador = _denominador * otro._denominador;
	tmp.simplifica();
	return tmp;
}

Fraccion Fraccion::operator-(Fraccion otro) const
{
	Fraccion tmp;
	tmp._numerador = _numerador * otro._denominador -
                  _denominador * otro._numerador;
	tmp._denominador = _denominador * otro._denominador;
	tmp.simplifica();
	return tmp;
}

Fraccion Fraccion::operator*(Fraccion otro) const
{
	Fraccion tmp;
	tmp._numerador = _numerador * otro._numerador;
	tmp._denominador = _denominador * otro._denominador;
	tmp.simplifica();
	return tmp;
}

Fraccion Fraccion::operator/(Fraccion otro) const
{
	Fraccion tmp;
	tmp._numerador = _numerador * otro._denominador;
	tmp._denominador = _denominador * otro._numerador;
	tmp.simplifica();
	return tmp;
}

bool Fraccion::operator==(Fraccion otro) const
{
	return (_numerador*otro._denominador) == (_denominador*otro._numerador);
}

bool Fraccion::operator!=(Fraccion otro) const
{
	return (_numerador*otro._denominador) != (_denominador*otro._numerador);
}

bool Fraccion::operator<(Fraccion otro) const
{
	double izq, der;
	izq = double(_numerador) / double(_denominador);
	der = double(otro._numerador) / double(otro._denominador);
	return izq < der;
}

bool Fraccion::operator<=(Fraccion otro) const
{
	double izq, der;
	izq = double(_numerador) / double(_denominador);
	der = double(otro._numerador) / double(otro._denominador);
	return izq <= der;
}

bool Fraccion::operator>(Fraccion otro) const
{
	double izq, der;
	izq = double(_numerador) / double(_denominador);
	der = double(otro._numerador) / double(otro._denominador);
	return izq > der;
}

bool Fraccion::operator>=(Fraccion otro) const
{
	double izq, der;
	izq = double(_numerador) / double(_denominador);
	der = double(otro._numerador) / double(otro._denominador);
	return izq >= der;
}

Fraccion& Fraccion::operator++()
{
	_numerador++;
	_denominador++;
	simplifica();
	return *this;
}

Fraccion& Fraccion::operator--()
{
	_numerador--;
	_denominador--;
	simplifica();
	return *this;
}

Fraccion Fraccion::operator-()
{
  Fraccion tmp;
	tmp._numerador = -_numerador;
  tmp._denominador = _denominador;
	return tmp;
}

Fraccion& Fraccion::operator+=(Fraccion otro)
{
	_numerador = _numerador * otro._denominador +
                  _denominador * otro._numerador;
	_denominador = _denominador * otro._denominador;
	simplifica();
	return *this;
}

Fraccion& Fraccion::operator-=(Fraccion otro)
{
	_numerador = _numerador * otro._denominador -
                  _denominador * otro._numerador;
	_denominador = _denominador * otro._denominador;
	simplifica();
	return *this;
}

void Fraccion::mostrar() const
{
  cout << _numerador << '/' << _denominador;
}

