// Clase FechaHora - Implementacin "fechahora.cpp"

#include "fechahora.h"

long int FechaHora::SPD = 86400;

FechaHora::FechaHora() { }

FechaHora::FechaHora(const FechaHora& otra) {
  _fecha = otra._fecha;
  _hora = otra._hora;
}

FechaHora& FechaHora::operator=(const FechaHora& otra) {
  _fecha = otra._fecha;
  _hora = otra._hora;
  return *this;
}
FechaHora::~FechaHora() { }

// Otros constructores:

FechaHora::FechaHora(Fecha f) : _fecha(f) { }

FechaHora::FechaHora(Hora h) : _hora(h) { }

FechaHora::FechaHora(Fecha f, Hora h) : _fecha(f), _hora(h) { }

// Accedentes y mutadores:

Fecha FechaHora::fecha() const { return _fecha; }

Hora FechaHora::hora() const { return _hora; }

void FechaHora::fecha(Fecha f) { _fecha = f; }

void FechaHora::hora(Hora h) { _hora = h; }

// Operadores aritmticos:

long int FechaHora::operator-(FechaHora otra) const {
// Segundos entre la FechaHora y la proporcionada
  return (_fecha - otra._fecha) * SPD + (_hora - otra._hora);
}

long int FechaHora::operator-(Hora otra) const {
// Segundos entre la hora y la proporcionada
  return _hora - otra;
}

long int FechaHora::operator-(Fecha otra) const {
// Das entre la fecha y la proporcionada
  return _fecha - otra;
}

FechaHora FechaHora::operator+(long int seg) const {
// + segundos
  FechaHora fh;
  fh._fecha = _fecha + seg / SPD; // incremento en das
  seg = seg % SPD; // segundos restantes
  fh._hora = _hora + seg;
  if(fh._hora < _hora) fh._fecha++; // cambio de da?
  return fh;
}

FechaHora FechaHora::operator-(long int seg) const {
// - segundos
  FechaHora fh;
  fh._fecha = _fecha - seg / SPD; // decremento en das
  seg = seg % SPD; // segundos restantes
  fh._hora = _hora - seg;
  if(fh._hora > _hora) fh._fecha--; // cambio de da?
  return fh;
}

FechaHora& FechaHora::operator++() { // Un segundo ms
  _hora++;
  if(_hora == Hora(0,0,0)) _fecha++; // cambio de fecha
  return *this;
}

FechaHora& FechaHora::operator--() { // Un segundo menos
  if(_hora == Hora(0,0,0)) _fecha--; // cambio de fecha
  _hora--;
  return *this;
}

// Operadores relacionales:

bool FechaHora::operator==(FechaHora otra) const {
  return (_fecha == otra._fecha) && (_hora == otra._hora);
}

bool FechaHora::operator!=(FechaHora otra) const {
  return (_fecha != otra._fecha) || (_hora != otra._hora);
}

bool FechaHora::operator<(FechaHora otra) const {
  return (_fecha < otra._fecha) ||
         (_fecha == otra._fecha) && (_hora < otra._hora);
}

bool FechaHora::operator<=(FechaHora otra) const {
  return (_fecha < otra._fecha) ||
         (_fecha == otra._fecha) && (_hora <= otra._hora);
}

bool FechaHora::operator>(FechaHora otra) const {
  return (_fecha > otra._fecha) ||
         (_fecha == otra._fecha) && (_hora > otra._hora);
}

bool FechaHora::operator>=(FechaHora otra) const {
  return (_fecha > otra._fecha) ||
         (_fecha == otra._fecha) && (_hora >= otra._hora);
}

// Otros mtodos:

long int FechaHora::aDias(long int seg) { return seg / SPD; }
// Transforma esos segundos en das (completos)

long int FechaHora::aSegundos(long int dias) { return dias * SPD; }
// Transforma esos das en segundos

string FechaHora::diaSemana() const {
// Devuelve el nombre del da de la semana
  return _fecha.diaSemana();
}

string FechaHora::completa() const {
// Ejemplo: "lunes 4 de noviembre de 2002 17:35:12"
  return _fecha.completa() + " " + _hora.hora();
}

string FechaHora::reducida() const {
// Ejemplo: "04/11/2002 17:35:12"
  return _fecha.reducida() + " " + _hora.hora();
}

void FechaHora::leer() {
  _fecha.leer();
  _hora.leer();
}

