// Clase FechaHora con herencia - Implementacin "fechahora.cpp"

#include "fechahora.h"

long FechaHora::SPD = 86400;

FechaHora::FechaHora() { }

FechaHora::FechaHora(const FechaHora& otra)
  : Fecha(otra), Hora(otra) { }

FechaHora& FechaHora::operator=(const FechaHora& otra) {
  Fecha::operator=(otra);
  Hora::operator=(otra);
  return *this;
}

FechaHora::~FechaHora() { }

// Otros constructores:

FechaHora::FechaHora(Fecha f) : Fecha(f) { }

FechaHora::FechaHora(Hora h) : Hora(h) { }

FechaHora::FechaHora(Fecha f, Hora h) : Fecha(f), Hora(h) { }

Fecha FechaHora::fecha() const
{ return Fecha (dia(), mes(), anio()); }

Hora FechaHora::hora() const
{ return Hora(horas(), minutos(), segundos()); }

void FechaHora::fecha(Fecha f) {
  dia(f.dia());
  mes(f.mes());
  anio(f.anio());
}

void FechaHora::hora(Hora h) {
  horas(h.horas());
  minutos(h.minutos());
  segundos(h.segundos());
}

// Operadores aritmticos:

long int FechaHora::operator-(FechaHora otra) const {
// Segundos entre la FechaHora y la proporcionada
  return Fecha::operator-(otra) * SPD + Hora::operator-(otra);
}

FechaHora FechaHora::operator+(long int seg) const { // + segundos
  Fecha f = fecha();
  Hora h = hora();
  f = f + seg / SPD; // incremento en das
  seg = seg % SPD; // segundos restantes
  h = h + seg;
  if(h < hora()) f++; // cambio de da?
  return FechaHora(f, h);
}

FechaHora FechaHora::operator-(long int seg) const { // - segundos
  Fecha f = fecha();
  Hora h = hora();
  f = f - seg / SPD; // decremento en das
  seg = seg % SPD; // segundos restantes
  h = h - seg;
  if(h > hora()) f--; // cambio de da?
  return FechaHora(f, h);
}

FechaHora& FechaHora::operator++() { // Un segundo ms
  Hora::operator++();
  if(hora() == Hora(0,0,0)) Fecha::operator++(); // Cambio de da
  return *this;
}

FechaHora& FechaHora::operator--() { // Un segundo menos
  if(hora() == Hora(0,0,0)) Fecha::operator--(); // Cambio de da
  Hora::operator--();
  return *this;
}

// Operadores relacionales:

bool FechaHora::operator==(FechaHora otra) const {
  return Fecha::operator==(otra) &&
         Hora::operator==(otra);
}

bool FechaHora::operator!=(FechaHora otra) const {
  return Fecha::operator!=(otra) ||
         Hora::operator!=(otra);
}

bool FechaHora::operator<(FechaHora otra) const {
  return Fecha::operator<(otra) ||
         Fecha::operator==(otra) && Hora::operator<(otra);
}

bool FechaHora::operator<=(FechaHora otra) const {
  return Fecha::operator<(otra) ||
         Fecha::operator==(otra) && Hora::operator<=(otra);
}

bool FechaHora::operator>(FechaHora otra) const {
  return Fecha::operator>(otra) ||
         Fecha::operator==(otra) && Hora::operator>(otra);
}

bool FechaHora::operator>=(FechaHora otra) const {
  return Fecha::operator>(otra) ||
         Fecha::operator==(otra) && Hora::operator>=(otra);
}

// Otros mtodos:

long int FechaHora::aDias(long int seg) { return seg / SPD; }

long int FechaHora::aSegundos(long int dias)
{ return dias * SPD; } // Transforma esos das en segundos

string FechaHora::completa() const {
// Cadena con la fecha completa seguida de la hora
  return Fecha::completa() + " " + Hora::hora();
}

string FechaHora::reducida() const {
// Cadena con la fecha reducida seguida de la hora
  return Fecha::reducida() + " " + Hora::hora();
}

void FechaHora::leer() {
  Fecha::leer();
  Hora::leer();
}

