// Clase Fecha - Implementacin "fecha.cpp"

#include "fecha.h"

int Fecha::dias[12] = {31,28,31,30,31,30,31,31,30,31,30,31};

// Mtodos privados:

void Fecha::valida(){
  if(_anio < 1900 || _anio > 2050) _anio = 1900;
  if(_mes < 1 || _mes > 12) _mes = 1;
  if(_dia < 1) _dia = 1;
  else {
    if(_mes == 2) {
      if(bisiesto()) { if(_dia > 29) _dia = 1; }
      else if(_dia > 28) _dia = 1;
    }
    else if(_dia > dias[_mes - 1]) _dia = 1;
  }
}

long int Fecha::diasDesdeInicio() const {
  long int total;
  int anios, m, bisiestos = 0;
  anios = _anio - 1900;
  total = anios * 365;
  bisiestos = (anios + 3) / 4;
  total += bisiestos;
  for(m = 1; m < _mes; m++) total += dias[m - 1];
  if(_mes > 2 && bisiesto()) total++;
  total += _dia; // Total de das desde 1/1/1900 hasta la fecha.
    // Incluyendo ambas fechas (se ha de restar una).
  total--;
  return total;
}

Fecha& Fecha::fechaTras(long int d) {
  int diasAnio, diasMes;
  _dia = 1;
  _mes = 1;
  _anio = 1900;
  if(d < 0) return *this;
  while(d > 0) {
    if(bisiesto()) diasAnio = 366;
    else diasAnio = 365;
    if(d >= diasAnio) {
      d -= diasAnio;
      _anio++;
      if(_anio == 2051) {
        _dia = 31;
        _mes = 12;
        _anio = 2050;
        return *this;
      }
    }
    else {
      if(_mes == 2 && bisiesto()) diasMes = 29;
      else diasMes = dias[_mes - 1];
      if(d >= diasMes) {
        d -= diasMes;
        _mes++;
      }
      else {
        _dia += d;
        d = 0;
      }
    }
  }
  return *this;
}

// Forma cannica ortodoxa:

Fecha::Fecha(int d, int m, int a)
  : _dia(d), _mes(m), _anio(a) { valida(); }

Fecha::Fecha(const Fecha& otra) {
  _dia = otra._dia;
  _mes = otra._mes;
  _anio = otra._anio;
}

Fecha& Fecha::operator=(const Fecha& otra) {
  _dia = otra._dia;
  _mes = otra._mes;
  _anio = otra._anio;
  return *this;
}

Fecha::~Fecha() { }

// Accedentes y mutadores:

int Fecha::dia() const { return _dia; }

int Fecha::mes() const { return _mes; }

int Fecha::anio() const { return _anio; }

void Fecha::dia(int d) { _dia = d; valida(); }

void Fecha::mes(int m) { _mes = m; valida(); }

void Fecha::anio(int a) { _anio = a; valida(); }

// Operadores aritmticos:

long int Fecha::operator-(Fecha otra) const
{ return diasDesdeInicio() - otra.diasDesdeInicio(); }

Fecha Fecha::operator+(long int dias) const
{ return Fecha().fechaTras(diasDesdeInicio() + dias); }

Fecha Fecha::operator-(long int dias) const
{ return Fecha().fechaTras(diasDesdeInicio() - dias); }

Fecha& Fecha::operator+=(long int dias)
{ fechaTras(diasDesdeInicio() + dias); return *this; }

Fecha& Fecha::operator-=(long int dias)
{ fechaTras(diasDesdeInicio() - dias); return *this; }

Fecha& Fecha::operator++()
{ fechaTras(diasDesdeInicio()+1); return *this; }

Fecha& Fecha::operator++(int)
{ fechaTras(diasDesdeInicio()+1); return *this; }

Fecha& Fecha::operator--()
{ fechaTras(diasDesdeInicio()-1); return *this; }

Fecha& Fecha::operator--(int)
{ fechaTras(diasDesdeInicio()-1); return *this; }

// Operadores relacionales:

bool Fecha::operator==(Fecha otra) const
{ return diasDesdeInicio() == otra.diasDesdeInicio(); }

bool Fecha::operator!=(Fecha otra) const
{ return diasDesdeInicio() != otra.diasDesdeInicio(); }

bool Fecha::operator<(Fecha otra) const
{ return diasDesdeInicio() < otra.diasDesdeInicio(); }

bool Fecha::operator<=(Fecha otra) const
{ return diasDesdeInicio() <= otra.diasDesdeInicio(); }

bool Fecha::operator>(Fecha otra) const
{ return diasDesdeInicio() > otra.diasDesdeInicio(); }

bool Fecha::operator>=(Fecha otra) const
{ return diasDesdeInicio() >= otra.diasDesdeInicio(); }

// Resto de mtodos:

bool Fecha::bisiesto() const
{ return (_anio % 4) == 0; }

string Fecha::diaSemana() const {
  int nd = diasDesdeInicio() % 7;
  switch(nd) {
    case 1: return("lunes");
    case 2: return("martes");
    case 3: return("miercoles");
    case 4: return("jueves");
    case 5: return("viernes");
    case 6: return("sabado");
    default: return("domingo");
  }
}

string Fecha::completa() const {
  string fc(diaSemana());
  fc += " ";
  char d[3];
  itoa(_dia, d, 10);
  string tmp(d);
  fc += tmp + " de ";
  switch(_mes) {
    case  1: tmp = "enero"; break;
    case  2: tmp = "febrero"; break;
    case  3: tmp = "marzo"; break;
    case  4: tmp = "abril"; break;
    case  5: tmp = "mayo"; break;
    case  6: tmp = "junio"; break;
    case  7: tmp = "julio"; break;
    case  8: tmp = "agosto"; break;
    case  9: tmp = "septiembre"; break;
    case 10: tmp = "octubre"; break;
    case 11: tmp = "noviembre"; break;
    case 12: tmp = "diciembre"; break;
  }
  fc += tmp + " de ";
  char a[5];
  itoa(_anio, a, 10);
  tmp = a;
  fc += tmp;
  return fc;
}

string Fecha::reducida() const {
  string fr;
  if(_dia < 10) fr += "0";
  char d[3];
  itoa(_dia, d, 10);
  string tmp(d);
  fr += tmp + "/";
  if(_mes < 10) fr += "0";
  char m[3];
  itoa(_mes, m, 10);
  tmp = m;
  fr += tmp + "/";
  char a[5];
  itoa(_anio, a, 10);
  tmp = a;
  fr += tmp;
  return fr;
}

void Fecha::leer() {
  cout << "Dia (1-31): "; cin >> _dia;
  cout << "Mes (1-12): "; cin >> _mes;
  cout << "Anio (1900-2050): "; cin >> _anio;
  valida();
}

